@@ -46,53 +46,14 @@ module Array = struct
        else true
      in check (pred n))
 
-  let fforall f xs =
-    let rec loop_outer i =
-      if i >= 0 then
-        let x = xs.(i) in
-        let n = Array.length x in
-        let rec loop_inner j =
-          if j >= 0 then f x.(j) && loop_inner (pred j)
-          else loop_outer (pred i)
-        in loop_inner (pred n)
-      else true
-    in
-    let n = Array.length xs in
-    loop_outer (pred n)
-
-  let fforall2 f xs ys =
-    let rec loop_outer i =
-      if i >= 0 then
-        let x = xs.(i) and y = ys.(i) in
-        let n = Array.length x in
-        n = Array.length y &&
-        let rec loop_inner j =
-          if j >= 0 then f x.(j) y.(j) && loop_inner (pred j)
-          else loop_outer (pred i)
-        in loop_inner (pred n)
-      else true
-    in
-    let n = Array.length xs in
-    n = Array.length ys &&
-    loop_outer (pred n)
-
-  let fforall3 f xs ys zs =
-    let rec loop_outer i =
-      if i >= 0 then
-        let x = xs.(i) and y = ys.(i) and z = zs.(i) in
-        let n = Array.length x in
-        n = Array.length y &&
-        n = Array.length z &&
-        let rec loop_inner j =
-          if j >= 0 then f x.(j) y.(j) z.(j) && loop_inner (pred j)
-          else loop_outer (pred i)
-        in loop_inner (pred n)
-      else true
-    in
-    let n = Array.length xs in
-    n = Array.length ys &&
-    n = Array.length zs &&
-    loop_outer (pred n)
+  let forall3 f a b c =
+    let n = Array.length a in
+    n = Array.length b &&
+    n = Array.length c &&
+    (let rec check i =
+       if i >= 0 then f a.(i) b.(i) c.(i) && check (pred i)
+       else true
+     in check (pred n))
 
   let map2 f a b =
     Array.mapi (fun i ai -> f ai b.(i)) a
@@ -100,30 +61,6 @@ module Array = struct
   let map3 f a b c =
     Array.mapi (fun i ai -> f ai b.(i) c.(i)) a
 
-  let mmap f a =
-    Array.map (fun ai ->
-      Array.map f ai
-    ) a
-
-  let mmap2 f a b =
-    Array.mapi (fun i ai ->
-      let bi = b.(i) in
-      Array.mapi (fun j aj ->
-        f aj bi.(j)
-      ) ai
-    ) a
-
-  let mmap3 f a b c =
-    Array.mapi (fun i ai ->
-      let bi = b.(i) and ci = c.(i) in
-      Array.mapi (fun j aj ->
-        f aj bi.(j) ci.(j)
-      ) ai
-    ) a
-
-  let ssplit a =
-    mmap fst a, mmap snd a
-
   let findi f a =
     let n = Array.length a in
     let rec loop i =
@@ -134,6 +71,9 @@ module Array = struct
       else None
     in loop 0
 
+  let split a =
+    Array.map fst a, Array.map snd a
+
 end
 
 module String = struct
@@ -175,6 +115,61 @@ module Option = struct
     | None -> None
 end
 
+module Shape = struct
+  type 'a t =
+    | SAtomic of 'a
+    | SArray of 'a t array
+
+  let of_array x =
+    SArray (Array.map (fun x -> SAtomic x) x)
+
+  let to_array = function
+    | SAtomic _ -> invalid_arg "Shape.to_array"
+    | SArray xs ->
+       Array.map (function
+           | SAtomic x -> x
+           | SArray _ -> invalid_arg "Shape.to_array"
+         ) xs
+
+  let to_shape_array = function
+    | SAtomic _ -> invalid_arg "Shape.to_shape_array"
+    | SArray xs -> xs
+
+  let rec map f = function
+    | SAtomic x -> SAtomic (f x)
+    | SArray x -> SArray (Array.map (map f) x)
+
+  let rec map2 f a b =
+    match a, b with
+    | SAtomic x, SAtomic y -> SAtomic (f x y)
+    | SArray x, SArray y -> SArray (Array.map2 (map2 f) x y)
+    | _, _ -> invalid_arg "Shape.map2"
+
+  let rec flatten = function
+    | SAtomic x -> [x]
+    | SArray xs -> Array.map flatten xs |> Array.to_list |> List.flatten
+
+  let split x =
+    map fst x, map snd x
+
+  let rec forall p = function
+    | SAtomic x -> p x
+    | SArray x -> Array.forall (forall p) x
+
+  let rec forall2 p x y =
+    match x, y with
+    | SAtomic x, SAtomic y -> p x y
+    | SArray x, SArray y -> Array.forall2 (forall2 p) x y
+    | _, _ -> invalid_arg "Shape.forall2"
+
+  let rec forall3 p x y z =
+    match x, y, z with
+    | SAtomic x, SAtomic y, SAtomic z -> p x y z
+    | SArray x, SArray y, SArray z -> Array.forall3 (forall3 p) x y z
+    | _, _, _ -> invalid_arg "Shape.forall3"
+
+end
+
 let save_to filename writer x =
   let oc = open_out filename in
   let ob = Bi_outbuf.create_channel_writer oc in
@@ -215,6 +210,7 @@ let bytes_to_sample q =
 
 module DirectRandom = struct
   type 'a t = 'a
+  let yield () = ()
   let return x = x
   let bind x f = f x
   let fail e = raise e
