@@ -19,28 +19,19 @@
 (*  <http://www.gnu.org/licenses/>.                                       *)
 (**************************************************************************)
 
-open Signatures
+open Signatures_core
 
 module Array : sig
   include module type of Array
   val exists : ('a -> bool) -> 'a array -> bool
   val forall : ('a -> bool) -> 'a array -> bool
   val forall2 : ('a -> 'b -> bool) -> 'a array -> 'b array -> bool
-  val fforall : ('a -> bool) -> 'a array array -> bool
-  val fforall2 : ('a -> 'b -> bool) ->
-    'a array array -> 'b array array -> bool
-  val fforall3 : ('a -> 'b -> 'c -> bool) ->
-    'a array array -> 'b array array -> 'c array array -> bool
+  val forall3 : ('a -> 'b -> 'c -> bool) -> 'a array -> 'b array -> 'c array -> bool
   val map2 : ('a -> 'b -> 'c) -> 'a array -> 'b array -> 'c array
   val map3 : ('a -> 'b -> 'c -> 'd) ->
     'a array -> 'b array -> 'c array -> 'd array
-  val mmap : ('a -> 'b) -> 'a array array -> 'b array array
-  val mmap2 : ('a -> 'b -> 'c) ->
-    'a array array -> 'b array array -> 'c array array
-  val mmap3 : ('a -> 'b -> 'c -> 'd) ->
-    'a array array -> 'b array array -> 'c array array -> 'd array array
-  val ssplit : ('a * 'b) array array -> 'a array array * 'b array array
   val findi : (int -> 'a -> 'b option) -> 'a array -> 'b option
+  val split : ('a * 'b) array -> 'a array * 'b array
 end
 
 module String : sig
@@ -60,6 +51,22 @@ module Option : sig
   val map : ('a -> 'b) -> 'a option -> 'b option
 end
 
+module Shape : sig
+  type 'a t =
+    | SAtomic of 'a
+    | SArray of 'a t array
+  val of_array : 'a array -> 'a t
+  val to_array : 'a t -> 'a array
+  val to_shape_array : 'a t -> 'a t array
+  val map : ('a -> 'b) -> 'a t -> 'b t
+  val map2 : ('a -> 'b -> 'c) -> 'a t -> 'b t -> 'c t
+  val flatten : 'a t -> 'a list
+  val split : ('a * 'b) t -> 'a t * 'b t
+  val forall : ('a -> bool) -> 'a t -> bool
+  val forall2 : ('a -> 'b -> bool) -> 'a t -> 'b t -> bool
+  val forall3 : ('a -> 'b -> 'c -> bool) -> 'a t -> 'b t -> 'c t -> bool
+end
+
 val save_to : string -> (Bi_outbuf.t -> 'a -> unit) -> 'a -> unit
 
 val compare_b64 : string -> string -> int
