@@ -20,6 +20,7 @@
 (**************************************************************************)
 
 open Platform
+open Common
 
 type number = Z.t
 type uuid
@@ -28,3 +29,7 @@ val min_uuid_length : int
 
 val uuid_of_raw_string : string -> uuid
 val raw_string_of_uuid : uuid -> string
+
+type 'a shape = 'a Shape.t =
+  | SAtomic of 'a
+  | SArray of 'a shape array
