@@ -21,6 +21,8 @@
 
 open Js_of_ocaml
 
+let debug x = Firebug.console##log (Js.string x)
+
 module Sjcl = struct
   open Js
 
@@ -181,6 +183,7 @@ module Jsbn = struct
       method add : bigint t -> bigint t meth
       method subtract : bigint t -> bigint t meth
       method multiply : bigint t -> bigint t meth
+      method divide : bigint t -> bigint t meth
       method _mod : bigint t -> bigint t meth
       method intValue : int meth
       method toString : js_string t meth
@@ -189,6 +192,9 @@ module Jsbn = struct
       method modInverse : bigint t -> bigint t meth
       method bitLength : int meth
       method isProbablePrime : int -> int meth
+      method shiftLeft : int -> bigint t meth
+      method shiftRight : int -> bigint t meth
+      method _and : bigint t -> bigint t meth
     end
 
   class type lib =
@@ -213,6 +219,7 @@ module Z = struct
   let ( + ) x y = x##add y
   let ( - ) x y = x##subtract y
   let ( * ) x y = x##multiply y
+  let ( / ) x y = x##divide y
   let ( mod ) x y = x##_mod y
 
   let to_int x = x##intValue
@@ -240,4 +247,8 @@ module Z = struct
       then loop (res * z256 + of_int (int_of_char x.[i])) (pred i)
       else res
     in loop zero (pred n)
+
+  let shift_left x n = x##shiftLeft n
+  let shift_right x n = x##shiftRight n
+  let logand x y = x##_and y
 end
