@@ -33,14 +33,16 @@ let stream_to_list s =
 let lines_of_file fname =
   let ic = open_in fname in
   Stream.from (fun _ ->
-    try Some (input_line ic)
-    with End_of_file -> close_in ic; None
-  )
+      match input_line ic with
+      | line -> Some line
+      | exception End_of_file -> close_in ic; None
+    )
 
 let lines_of_stdin () =
   Stream.from (fun _ ->
-      try Some (input_line stdin)
-      with End_of_file -> None
+      match input_line stdin with
+      | line -> Some line
+      | exception End_of_file -> None
     )
 
 let string_of_file f =
@@ -86,12 +88,11 @@ let get_mandatory_opt name = function
   | None -> failcmd "%s is mandatory" name
 
 let wrap_main f =
-  try
-    let () = f () in `Ok ()
-  with
-  | Cmdline_error e -> `Error (true, e)
-  | Failure e -> `Error (false, e)
-  | e -> `Error (false, Printexc.to_string e)
+  match f () with
+  | () -> `Ok ()
+  | exception Cmdline_error e -> `Error (true, e)
+  | exception Failure e -> `Error (false, e)
+  | exception e -> `Error (false, Printexc.to_string e)
 
 module type CMDLINER_MODULE = sig
   val cmds : (unit Cmdliner.Term.t * Cmdliner.Term.info) list
@@ -297,8 +298,7 @@ module Election : CMDLINER_MODULE = struct
     let get_threshold () =
       let file = "threshold.json" in
       Printf.eprintf "I: loading %s...\n%!" file;
-      try Some (string_of_file (X.dir / file))
-      with _ -> None
+      try Some (string_of_file (X.dir / file)) with _ -> None
 
     let get_public_keys () =
       load_from_file (fun x -> x) (X.dir/"public_keys.jsons") |>
@@ -307,14 +307,20 @@ module Election : CMDLINER_MODULE = struct
     let get_public_creds () =
       let file = "public_creds.txt" in
       Printf.eprintf "I: loading %s...\n%!" file;
-      try Some (lines_of_file (X.dir / file))
-      with _ -> None
+      try Some (lines_of_file (X.dir / file)) with _ -> None
 
     let get_ballots () =
       let file = "ballots.jsons" in
       Printf.eprintf "I: loading %s...\n%!" file;
-      try Some (lines_of_file (X.dir / file))
-      with _ -> None
+      try Some (lines_of_file (X.dir / file)) with _ -> None
+
+    let get_shuffles () =
+      let file = "shuffles.jsons" in
+      if Sys.file_exists (X.dir / file) then (
+        Printf.eprintf "I: loading %s...\n%!" file;
+        try Some (lines_of_file (X.dir / file))
+        with _ -> None
+      ) else None
 
     let get_result () =
       load_from_file (fun x -> x) (X.dir/"result.json") |> function
@@ -393,10 +399,14 @@ module Election : CMDLINER_MODULE = struct
           | Some factors -> factors
           | None -> failwith "cannot load partial decryptions"
         in
+        let result = X.validate factors in
         let oc = open_out (dir/"result.json") in
-        output_string oc (X.validate factors);
+        output_string oc result;
         output_char oc '\n';
         close_out oc
+      | `Shuffle ->
+         let s = X.shuffle_ciphertexts () in
+         print_endline s
       end;
       if cleanup then rm_rf dir
     )
@@ -479,7 +489,17 @@ module Election : CMDLINER_MODULE = struct
     Term.(ret (pure main $ url_t $ optdir_t $ pure `Validate)),
     Term.info "validate" ~doc ~man
 
-  let cmds = [vote_cmd; verify_cmd; decrypt_cmd; tdecrypt_cmd; validate_cmd]
+  let shuffle_cmd =
+    let doc = "shuffle ciphertexts" in
+    let man = [
+        `S "DESCRIPTION";
+        `P "This command shuffles non-homomorphic ciphertexts and prints on standard output the shuffle proof and the shuffled ciphertexts.";
+      ] @ common_man
+    in
+    Term.(ret (pure main $ url_t $ optdir_t $ pure `Shuffle)),
+    Term.info "shuffle" ~doc ~man
+
+  let cmds = [vote_cmd; verify_cmd; decrypt_cmd; tdecrypt_cmd; validate_cmd; shuffle_cmd]
 
 end
 
