@@ -4,6 +4,7 @@ module type PARAMS = sig
   val get_threshold : unit -> string option
   val get_public_creds : unit -> string Stream.t option
   val get_ballots : unit -> string Stream.t option
+  val get_shuffles : unit -> string Stream.t option
   val get_result : unit -> string option
   val print_msg : string -> unit
 end
@@ -14,6 +15,7 @@ module type S = sig
   val tdecrypt : string -> string -> string
   val validate : string list -> string
   val verify : unit -> unit
+  val shuffle_ciphertexts : unit -> string
 end
 
 val make : (module PARAMS) -> (module S)
