@@ -27,30 +27,13 @@ open Signatures
 open Common
 open Tool_js_common
 
-let prng = lazy (pseudo_rng (random_string secure_rng 16))
-
-module LwtJsRandom = struct
-  type 'a t = unit -> 'a Lwt.t
-  let return x () = Lwt.return x
-  let bind x f () = Lwt.bind (x ()) (fun y -> f y ())
-  let fail x () = Lwt.fail x
-
-  let random q =
-    let size = Z.bit_length q / 8 + 1 in
-    fun () ->
-      let%lwt () = Lwt_js.yield () in
-      let r = random_string (Lazy.force prng) size in
-      Lwt.return Z.(of_bits r mod q)
-end
-
 let encryptBallot params cred plaintext () =
   let module P = (val params : ELECTION_DATA) in
   let module G = P.G in
   let module E = Election.Make (P) (LwtJsRandom) in
   let module CD = Credential.MakeDerive (G) in
   let sk = CD.derive P.election.e_params.e_uuid cred in
-  let%lwt randomness = E.make_randomness () () in
-  let%lwt b = E.create_ballot ~sk randomness plaintext () in
+  let%lwt b = E.create_ballot ~sk plaintext in
   let s = string_of_ballot G.write b in
   set_textarea "ballot" s;
   set_content "ballot_tracker" (sha256_b64 s);
@@ -71,11 +54,8 @@ let progress_step n =
     new_##.style##.fontWeight := Js.string "bold"
   in ()
 
-let rec createQuestionNode sk params question_div num_questions i prev (q, answers) next =
-  (* Create div element for the current question. [i] and [(q,
-     answers)] point to the current question. [List.rev prev @ [q,
-     answers] @ next] is the list of all questions. *)
-  let div = Dom_html.createDiv document in
+let appendHomomorphicQuestion div num_questions i q answers =
+  let open Question_h_t in
   let () =
     let c = Dom_html.createH2 document in
     let t = document##createTextNode (Js.string q.q_question) in
@@ -100,6 +80,7 @@ let rec createQuestionNode sk params question_div num_questions i prev (q, answe
   in
   let () =
     let choices = Dom_html.createDiv document in
+    choices##.className := Js.string "answer_div";
     let choices_divs = Array.mapi (fun i a ->
       let div = Dom_html.createDiv document in
       let checkbox = Dom_html.createInput ~_type:(Js.string "checkbox") document in
@@ -155,6 +136,128 @@ let rec createQuestionNode sk params question_div num_questions i prev (q, answe
        let total = Array.fold_left (+) 0 answers in
        check_min_max total
   in
+  check_constraints
+
+let appendNonHomomorphicQuestion div q answers =
+  let open Question_nh_t in
+  let () =
+    let c = Dom_html.createH2 document in
+    let t = document##createTextNode (Js.string q.q_question) in
+    Dom.appendChild c t;
+    Dom.appendChild div c
+  in
+  let inputs =
+    let choices = Dom_html.createDiv document in
+    choices##.className := Js.string "answer_div";
+    let choices_divs = Array.mapi (fun i a ->
+      let div = Dom_html.createDiv document in
+      let input = Dom_html.createInput document in
+      input##.size := 5;
+      input##.value := Js.string (string_of_int answers.(i));
+      Dom.appendChild div input;
+      let t = document##createTextNode (Js.string (" " ^ a)) in
+      input##.onchange :=
+        Dom_html.handler (fun _ ->
+            let x =
+              try
+                let x = int_of_string (Js.to_string input##.value) in
+                if x < 0 then raise Exit;
+                if x > 255 then raise Exit;
+                x
+              with _ ->
+                alert (get_content "alert_0_255");
+                answers.(i)
+            in
+            answers.(i) <- x;
+            input##.value := Js.string (string_of_int x);
+            Js._true
+          );
+      Dom.appendChild div t;
+      input, div
+    ) q.q_answers
+    in
+    for i = 0 to Array.length choices_divs - 1 do
+      Dom.appendChild choices (snd choices_divs.(i))
+    done;
+    Dom.appendChild div choices;
+    Array.map fst choices_divs
+  in
+  let check_constraints () =
+    let n = Array.length inputs - 1 in
+    let rec loop i =
+      if i < n then
+        let valid =
+          let x =
+            try Some (int_of_string (Js.to_string inputs.(i)##.value))
+            with _ -> None
+          in
+          match x with
+          | Some y -> y = answers.(i)
+          | None -> false
+        in
+        if not valid then (
+          alert (get_content "at_least_one_invalid");
+          false
+        ) else loop (i + 1)
+      else true
+    in
+    loop 0
+  in
+  check_constraints
+
+let appendHomomorphicSummary e a q =
+  let open Question_h_t in
+  let h = Dom_html.createH3 document in
+  let t = document##createTextNode (Js.string q.q_question) in
+  Dom.appendChild h t;
+  Dom.appendChild e h;
+  let ul = Dom_html.createUl document in
+  let checked = ref 0 in
+  Array.iteri (fun i a ->
+      if a > 0 then (
+        incr checked;
+        let li = Dom_html.createLi document in
+        let text = match q.q_blank with
+          | Some true -> if i = 0 then get_content "str_blank_vote" else q.q_answers.(i-1)
+          | _ -> q.q_answers.(i)
+        in
+        let t = document##createTextNode (Js.string text) in
+        Dom.appendChild li t;
+        Dom.appendChild ul li;
+      )
+    ) a;
+  if !checked = 0 then (
+    let t = document##createTextNode (Js.string @@ get_content "str_nothing") in
+    Dom.appendChild ul t
+  );
+  Dom.appendChild e ul
+
+let appendNonHomomorphicSummary e a q =
+  let open Question_nh_t in
+  let h = Dom_html.createH3 document in
+  let t = document##createTextNode (Js.string q.q_question) in
+  Dom.appendChild h t;
+  Dom.appendChild e h;
+  let ul = Dom_html.createUl document in
+  Array.iteri (fun i a ->
+      let li = Dom_html.createLi document in
+      let text = Printf.sprintf "%d (%s)" a q.q_answers.(i) in
+      let t = document##createTextNode (Js.string text) in
+      Dom.appendChild li t;
+      Dom.appendChild ul li;
+    ) a;
+  Dom.appendChild e ul
+
+let rec createQuestionNode sk params question_div num_questions i prev (q, answers) next =
+  (* Create div element for the current question. [i] and [(q,
+     answers)] point to the current question. [List.rev prev @ [q,
+     answers] @ next] is the list of all questions. *)
+  let div = Dom_html.createDiv document in
+  let nh, check_constraints =
+    match q with
+    | Question.Homomorphic q -> false, appendHomomorphicQuestion div num_questions i q answers
+    | Question.NonHomomorphic q -> true, appendNonHomomorphicQuestion div q answers
+  in
   let () =
     (* previous button *)
     let btns = Dom_html.createDiv document in
@@ -196,31 +299,9 @@ let rec createQuestionNode sk params question_div num_questions i prev (q, answe
           let () =
             document##getElementById (Js.string "pretty_choices") >>== fun e ->
             Array.iteri (fun i a ->
-                let q = all_questions.(i) in
-                let h = Dom_html.createH3 document in
-                let t = document##createTextNode (Js.string q.q_question) in
-                Dom.appendChild h t;
-                Dom.appendChild e h;
-                let ul = Dom_html.createUl document in
-                let checked = ref 0 in
-                Array.iteri (fun i a ->
-                    if a > 0 then (
-                      incr checked;
-                      let li = Dom_html.createLi document in
-                      let text = match q.q_blank with
-                        | Some true -> if i = 0 then get_content "str_blank_vote" else q.q_answers.(i-1)
-                        | _ -> q.q_answers.(i)
-                      in
-                      let t = document##createTextNode (Js.string text) in
-                      Dom.appendChild li t;
-                      Dom.appendChild ul li;
-                    )
-                  ) a;
-                if !checked = 0 then (
-                  let t = document##createTextNode (Js.string @@ get_content "str_nothing") in
-                  Dom.appendChild ul t
-                );
-                Dom.appendChild e ul;
+                match all_questions.(i) with
+                | Question.Homomorphic q -> appendHomomorphicSummary e a q
+                | Question.NonHomomorphic q -> appendNonHomomorphicSummary e a q
               ) all_answers
           in
           Lwt_js_events.async (encryptBallot params sk all_answers);
@@ -248,6 +329,14 @@ let rec createQuestionNode sk params question_div num_questions i prev (q, answe
     in
     Dom.appendChild div btns
   in
+  let () =
+    if nh then (
+      let d = Dom_html.createDiv document in
+      d##.style##.marginTop := Js.string "1em";
+      Dom.appendChild div d;
+      Dom.appendChild d (document##createTextNode (Js.string (get_content "warning_0_255")))
+    )
+  in
   div
 
 let addQuestions sk params qs =
@@ -255,7 +344,11 @@ let addQuestions sk params qs =
   let n = Array.length qs in
   let qs =
     Array.to_list qs |>
-      List.map (fun q -> q, Array.make (Election.question_length q) 0)
+      List.map (fun q ->
+          match q with
+          | Question.Homomorphic x -> q, Array.make (Question_h.question_length x) 0
+          | Question.NonHomomorphic x -> q, Array.make (Array.length x.Question_nh_t.q_answers) 0
+        )
   in
   match qs with
   | [] -> failwith "no questions"
@@ -315,28 +408,34 @@ let get_prefix str =
   let n = String.length str in
   if n >= 4 then String.sub str 0 (n-4) else str
 
-let get_url x =
+let get_uuid x =
   let n = String.length x in
   if n <= 1 || String.sub x 0 1 <> "#" then
     None
   else
     let args = Url.decode_arguments (String.sub x 1 (n-1)) in
-    List.assoc_opt "url" args
+    List.assoc_opt "uuid" args
 
-let load_url url =
+let load_uuid uuid =
   let open Lwt_xmlHttpRequest in
   Lwt.async (fun () ->
-      let%lwt raw = get (url ^ "election.json") in
-      let () = set_textarea "election_params" raw.content in
+      let%lwt raw =
+        let%lwt x = Printf.ksprintf get "elections/%s/election.json" uuid in
+        if x.code = 404 then (
+          let%lwt x = Printf.ksprintf get "draft/preview/%s/election.json" uuid in
+          Lwt.return x.content
+        ) else Lwt.return x.content
+      in
+      let () = set_textarea "election_params" raw in
       Lwt.return (run_handler loadElection ())
     )
 
-let load_url_handler _ =
-  (match get_textarea_opt "url" with
-   | Some url ->
-      let encoded = Url.encode_arguments ["url", url] in
+let load_uuid_handler _ =
+  (match get_textarea_opt "uuid" with
+   | Some uuid ->
+      let encoded = Url.encode_arguments ["uuid", uuid] in
       Dom_html.window##.location##.hash := Js.string encoded;
-      load_url url
+      load_uuid uuid
    | None -> ()
   ); Js._false
 
@@ -351,19 +450,19 @@ let load_params_handler _ =
 
 let onload_handler _ =
   let () =
-    document##getElementById (Js.string "load_url") >>== fun e ->
-    e##.onclick := Dom_html.handler load_url_handler
+    document##getElementById (Js.string "load_uuid") >>== fun e ->
+    e##.onclick := Dom_html.handler load_uuid_handler
   in
   let () =
     document##getElementById (Js.string "load_params") >>== fun e ->
     e##.onclick := Dom_html.handler load_params_handler;
   in
   let () =
-    match get_url (Js.to_string Dom_html.window##.location##.hash) with
+    match get_uuid (Js.to_string Dom_html.window##.location##.hash) with
     | None ->
        set_element_display "wait_div" "none";
        set_element_display "election_loader" "block";
-    | Some url -> load_url url
+    | Some uuid -> load_uuid uuid
   in Js._false
 
 let () = Dom_html.window##.onload := Dom_html.handler onload_handler
