@@ -20,6 +20,7 @@
 (**************************************************************************)
 
 open Js_of_ocaml
+open Platform
 open Common
 
 let document = Dom_html.document
@@ -91,9 +92,35 @@ let clear_content (e : #Dom.node Js.t) =
     ()
   done
 
+let clear_content_by_id id =
+  document##getElementById (Js.string id) >>== fun e ->
+  clear_content e
+
 let run_handler handler () =
   (try handler ()
    with e ->
      let msg = "Unexpected error: " ^ Printexc.to_string e in
      alert msg
   ); Js._false
+
+let get_params () =
+  let x = Js.to_string Dom_html.window##.location##.search in
+  let n = String.length x in
+  if n < 1 || x.[0] <> '?' then []
+  else Url.decode_arguments (String.sub x 1 (n-1))
+
+module LwtJsRandom : Signatures.RANDOM with type 'a t = 'a Lwt.t = struct
+  type 'a t = 'a Lwt.t
+  let yield = Lwt_js.yield
+  let return = Lwt.return
+  let bind = Lwt.bind
+  let fail = Lwt.fail
+
+  let prng = lazy (pseudo_rng (random_string secure_rng 16))
+
+  let random q =
+    let size = Z.bit_length q / 8 + 1 in
+    let%lwt () = Lwt_js.yield () in
+    let r = random_string (Lazy.force prng) size in
+    Lwt.return Z.(of_bits r mod q)
+end
