@@ -26,6 +26,32 @@ open Tool_js_common
 let return = Js.Opt.return
 let handler f = Dom_html.handler (fun e -> ignore (f e); Js._false)
 
+let hybrid_mode = ref false
+
+let q_answers = [| "Answer 1"; "Answer 2"; "Answer 3" |]
+
+let default_question_h =
+  let open Question_h_t in
+  Question.Homomorphic
+    {
+      q_question = "Question?";
+      q_min = 1;
+      q_max = 2;
+      q_blank = None;
+      q_answers;
+    }
+
+let default_question_nh =
+  let open Question_nh_t in
+  Question.NonHomomorphic
+    {
+      q_question = "Give a rank to each candidate (a number between 1 and 3)";
+      q_answers;
+    }
+
+let default_question () =
+  if !hybrid_mode then default_question_nh else default_question_h
+
 (* Getting the OCaml structure out of the DOM *)
 
 let extractAnswer a =
@@ -43,18 +69,9 @@ let extractQuestion q =
     p2##querySelector (Js.string selector) >>= fun x ->
     Dom_html.CoerceTo.input x >>= fun x ->
     let x = Js.to_string x##.value in
-    try return (int_of_string x)
-    with _ -> failwith (error_msg ^ ": " ^ x ^ ".")
+    try return (int_of_string x) with
+    | _ -> failwith (error_msg ^ ": " ^ x ^ ".")
   in
-  p2##querySelector (Js.string ".question_blank") >>= fun q_blank ->
-  Dom_html.CoerceTo.input q_blank >>= fun q_blank ->
-  let q_blank = if Js.to_bool q_blank##.checked then Some true else None in
-  numeric ".question_min" "Invalid minimum number of choices" >>= fun q_min ->
-  numeric ".question_max" "Invalid maximum number of choices" >>= fun q_max ->
-  if not (q_min <= q_max) then
-    failwith "Minimum number of choices must be less than or equal to maximum number of choices!";
-  if (q_max = 0) then
-    failwith "Maximum number of choices must be greater than 0!";
   let answers = p2##querySelectorAll (Js.string ".question_answer") in
   let q_answers =
     Array.init
@@ -63,9 +80,23 @@ let extractQuestion q =
        let a = answers##item (i) >>= extractAnswer in
        Js.Opt.get a (fun () -> failwith "extractQuestion"))
   in
-  if (q_max > Array.length q_answers) then
-    failwith "Maximum number of choices is greater than number of choices!";
-  return {q_question; q_blank; q_min; q_max; q_answers}
+  match Js.Opt.to_option (p2##querySelector (Js.string ".question_blank")) with
+  | Some q_blank ->
+     Dom_html.CoerceTo.input q_blank >>= fun q_blank ->
+     let q_blank = if Js.to_bool q_blank##.checked then Some true else None in
+     numeric ".question_min" "Invalid minimum number of choices" >>= fun q_min ->
+     numeric ".question_max" "Invalid maximum number of choices" >>= fun q_max ->
+     if not (q_min <= q_max) then
+       failwith "Minimum number of choices must be less than or equal to maximum number of choices!";
+     if (q_max = 0) then
+       failwith "Maximum number of choices must be greater than 0!";
+     if (q_max > Array.length q_answers) then
+       failwith "Maximum number of choices is greater than number of choices!";
+     let open Question_h_t in
+     return (Question.Homomorphic {q_question; q_blank; q_min; q_max; q_answers})
+  | None ->
+     let open Question_nh_t in
+     return (Question.NonHomomorphic {q_question; q_answers})
 
 let extractTemplate () =
   let t_name = get_input "election_name" in
@@ -117,8 +148,60 @@ let rec createAnswer a =
   Dom.appendChild container insert_btn;
   container
 
-let rec createQuestion q =
+let createHomomorphicQuestionPropDiv min max blank =
   let container = Dom_html.createDiv document in
+  let x = Dom_html.createDiv document in
+  let t = document##createTextNode (Js.string "The voter has to choose between ") in
+  Dom.appendChild x t;
+  let h_min = Dom_html.createInput document in
+  Dom.appendChild x h_min;
+  h_min##.className := Js.string "question_min";
+  h_min##.size := 5;
+  h_min##.value := Js.string (string_of_int min);
+  let t = document##createTextNode (Js.string " and ") in
+  Dom.appendChild x t;
+  let h_max = Dom_html.createInput document in
+  Dom.appendChild x h_max;
+  h_max##.className := Js.string "question_max";
+  h_max##.size := 5;
+  h_max##.value := Js.string (string_of_int max);
+  let t = document##createTextNode (Js.string " answers.") in
+  Dom.appendChild x t;
+  Dom.appendChild container x;
+  (* is blank allowed? *)
+  let x = Dom_html.createDiv document in
+  let h_blank = Dom_html.createInput ~_type:(Js.string "checkbox") document in
+  h_blank##.className := Js.string "question_blank";
+  h_blank##.checked := Js.(match blank with Some true -> _true | _ -> _false);
+  Dom.appendChild x h_blank;
+  let t = document##createTextNode (Js.string "Blank vote is allowed") in
+  Dom.appendChild x t;
+  Dom.appendChild container x;
+  container
+
+let default_props = None, 0, 1
+
+let gensym =
+  let counter = ref 0 in
+  fun () -> incr counter; !counter
+
+let deleteQuestion q =
+  q##.parentNode >>= fun x ->
+  Dom.removeChild x q;
+  return ()
+
+let rec createQuestion q =
+  let question, answers, props =
+    match q with
+    | Question.Homomorphic q ->
+       let open Question_h_t in
+       q.q_question, q.q_answers, Some (q.q_blank, q.q_min, q.q_max)
+    | Question.NonHomomorphic q ->
+       let open Question_nh_t in
+       q.q_question, q.q_answers, None
+  in
+  let container = Dom_html.createDiv document in
+  container##.className := Js.string "question";
   (* question text and remove/insert buttons *)
   let x = Dom_html.createDiv document in
   let t = document##createTextNode (Js.string "Question: ") in
@@ -127,7 +210,7 @@ let rec createQuestion q =
   Dom.appendChild x h_question;
   h_question##.className := Js.string "question_question";
   h_question##.size := 60;
-  h_question##.value := Js.string q.q_question;
+  h_question##.value := Js.string question;
   let remove_text = document##createTextNode (Js.string "Remove") in
   let remove_btn = Dom_html.createButton document in
   let f _ =
@@ -141,7 +224,7 @@ let rec createQuestion q =
   let insert_text = document##createTextNode (Js.string "Insert") in
   let insert_btn = Dom_html.createButton document in
   let f _ =
-    let x = createQuestion {q_question=""; q_blank=None; q_min=0; q_max=1; q_answers=[||]} in
+    let x = createQuestion (default_question ()) in
     container##.parentNode >>= fun p ->
     Dom.insertBefore p x (Js.some container);
     return ()
@@ -151,33 +234,57 @@ let rec createQuestion q =
   Dom.appendChild x insert_btn;
   Dom.appendChild container x;
   (* properties *)
+  let prop_div_h =
+    let blank, min, max =
+      match props with
+      | Some x -> x
+      | None -> default_props
+    in
+    createHomomorphicQuestionPropDiv min max blank
+  in
+  let type_div = Dom_html.createDiv document in
+  type_div##.style##.display := if !hybrid_mode then Js.string "block" else Js.string "none";
+  Dom.appendChild container type_div;
+  let prop_div_nh = Dom_html.createDiv document in
+  let nh_explain = Dom_html.createDiv document in
+  nh_explain##.className := Js.string "nh_explain";
+  Dom.appendChild nh_explain (document##createTextNode (Js.string "The voter has to enter an integer in front of each answer. The system will accept any integer between 0 and 255 but it is up to you to remove invalid ballots (score too high or candidates not properly ranked) at the end of the election."));
+  Dom.appendChild prop_div_nh nh_explain;
+  let _type = Js.string "radio" and name = Printf.ksprintf Js.string "type%d" (gensym ()) in
   let x = Dom_html.createDiv document in
-  let t = document##createTextNode (Js.string "The voter has to choose between ") in
-  Dom.appendChild x t;
-  let h_min = Dom_html.createInput document in
-  Dom.appendChild x h_min;
-  h_min##.className := Js.string "question_min";
-  h_min##.size := 5;
-  h_min##.value := Js.string (string_of_int q.q_min);
-  let t = document##createTextNode (Js.string " and ") in
-  Dom.appendChild x t;
-  let h_max = Dom_html.createInput document in
-  Dom.appendChild x h_max;
-  h_max##.className := Js.string "question_max";
-  h_max##.size := 5;
-  h_max##.value := Js.string (string_of_int q.q_max);
-  let t = document##createTextNode (Js.string " answers.") in
-  Dom.appendChild x t;
-  Dom.appendChild container x;
-  (* is blank allowed? *)
+  Dom.appendChild type_div x;
+  let cb_type_classical = Dom_html.createInput ~_type ~name document in
+  Dom.appendChild x cb_type_classical;
+  Dom.appendChild x (document##createTextNode (Js.string "Classical (selection of answers)"));
   let x = Dom_html.createDiv document in
-  let h_blank = Dom_html.createInput ~_type:(Js.string "checkbox") document in
-  h_blank##.className := Js.string "question_blank";
-  h_blank##.checked := Js.(match q.q_blank with Some true -> _true | _ -> _false);
-  Dom.appendChild x h_blank;
-  let t = document##createTextNode (Js.string "Blank vote is allowed") in
-  Dom.appendChild x t;
-  Dom.appendChild container x;
+  Dom.appendChild type_div x;
+  let cb_type = Dom_html.createInput ~_type ~name document in
+  cb_type##.className := Js.string "nonhomomorphic_tally";
+  (match props with
+   | Some _ ->
+      Dom.appendChild container prop_div_h;
+      cb_type_classical##.checked := Js._true
+   | None ->
+      Dom.appendChild container prop_div_nh;
+      cb_type##.checked := Js._true
+  );
+  let f =
+    handler
+      (fun _ ->
+        container##.parentNode >>= fun parent ->
+        if Js.to_bool cb_type##.checked then
+          Dom.replaceChild parent (createQuestion default_question_nh) container
+        else
+          Dom.replaceChild parent (createQuestion default_question_h) container;
+        return ()
+      )
+  in
+  cb_type##.onchange := f;
+  cb_type_classical##.onchange := f;
+  if not (Js.to_bool (Js.Unsafe.pure_js_expr "allow_nh")) then
+    cb_type##.disabled := Js._true;
+  Dom.appendChild x cb_type;
+  Dom.appendChild x (document##createTextNode (Js.string "Alternative (voters assign a number to each candidate)"));
   (* answers *)
   let h_answers = Dom_html.createDiv document in
   h_answers##.className := Js.string "question_answers";
@@ -186,7 +293,7 @@ let rec createQuestion q =
     (fun a ->
      let x = createAnswer a in
      Dom.appendChild h_answers x)
-    q.q_answers;
+    answers;
   (* button for adding answer *)
   let x = Dom_html.createDiv document in
   let b = Dom_html.createButton document in
@@ -248,7 +355,7 @@ let createTemplate template =
   let b = Dom_html.createButton document in
   let t = document##createTextNode (Js.string "Add a question") in
   let f _ =
-    let x = createQuestion {q_question=""; q_blank=None; q_min=0; q_max=1; q_answers=[||]} in
+    let x = createQuestion (default_question ()) in
     Dom.appendChild h_questions_div x
   in
   b##.onclick := handler f;
@@ -280,15 +387,39 @@ let createTemplate template =
   (* return *)
   container
 
+(* Handling of hybrid checkbox *)
+
+let handle_hybrid e _ =
+  hybrid_mode := Js.to_bool e##.checked;
+  let qs = document##querySelectorAll (Js.string ".question") in
+  for i = 0 to qs##.length do
+    ignore (qs##item i >>= deleteQuestion)
+  done;
+  document##getElementById (Js.string "election_questions") >>= fun qsdiv ->
+  Dom.appendChild qsdiv (createQuestion (default_question ()));
+  return ()
+
 (* Entry point *)
 
 let fill_interactivity _ =
   document##getElementById (Js.string "interactivity") >>= fun e ->
   let t = template_of_string (get_textarea "questions") in
+  let has_nh =
+    Array.exists
+      (function
+       | Question.NonHomomorphic _ -> true
+       | _ -> false
+      ) t.t_questions
+  in
+  hybrid_mode := has_nh;
   let div = createTemplate t in
   Dom.appendChild e div;
   document##querySelector (Js.string "form") >>= fun x ->
   x##.style##.display := Js.string "none";
+  document##getElementById (Js.string "hybrid_mode") >>= fun e ->
+  Dom_html.CoerceTo.input e >>= fun e ->
+  e##.checked := Js.bool !hybrid_mode;
+  e##.onchange := handler (handle_hybrid e);
   return ()
 
 let () =
