@@ -30,6 +30,7 @@ open Web_serializable_j
 module LwtRandom = struct
 
   type 'a t = 'a Lwt.t
+  let yield = Lwt_main.yield
   let return = Lwt.return
   let bind = Lwt.bind
   let fail = Lwt.fail
@@ -256,6 +257,7 @@ type add_account_error =
   | BadUsername
   | BadPassword of string
   | PasswordMismatch
+  | BadSpaceInPassword
 
 let b58_digits = "123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz"
 let prng = lazy (pseudo_rng (random_string secure_rng 16))
@@ -301,11 +303,10 @@ let send_email recipient subject body =
   let return_path = !Web_config.return_path in
   let sendmail = sendmail ?return_path in
   let rec loop () =
-    try%lwt
-      Lwt_preemptive.detach sendmail contents
-    with Unix.Unix_error (Unix.EAGAIN, _, _) ->
-      let%lwt () = Lwt_unix.sleep 1. in
-      loop ()
+    try%lwt Lwt_preemptive.detach sendmail contents with
+    | Unix.Unix_error (Unix.EAGAIN, _, _) ->
+       let%lwt () = Lwt_unix.sleep 1. in
+       loop ()
   in loop ()
 
 let split_identity x =
@@ -328,8 +329,7 @@ let languages_of_string x =
   Pcre.split x
 
 let pcre_exec_opt ~rex x =
-  try Some (Pcre.exec ~rex x)
-  with Not_found -> None
+  try Some (Pcre.exec ~rex x) with Not_found -> None
 
 let email_rex = "[A-Z0-9._%+-]+@[A-Z0-9.-]+\\.[A-Z]{2,}"
 
@@ -352,11 +352,9 @@ let extract_email =
   )
 
 let file_exists x =
-  try%lwt
-    let%lwt () = Lwt_unix.(access x [R_OK]) in
-    return true
-  with _ ->
-    return false
+  match%lwt Lwt_unix.(access x [R_OK]) with
+  | () -> return true
+  | exception _ -> return false
 
 let get_fname uuid x =
   match uuid with
@@ -366,10 +364,9 @@ let get_fname uuid x =
      !Web_config.spool_dir / raw_string_of_uuid uuid / x
 
 let read_file ?uuid x =
-  try%lwt
-    let%lwt lines = Lwt_io.lines_of_file (get_fname uuid x) |> Lwt_stream.to_list in
-    return (Some lines)
-  with _ -> return_none
+  match%lwt Lwt_io.lines_of_file (get_fname uuid x) |> Lwt_stream.to_list with
+  | lines -> return_some lines
+  | exception _ -> return_none
 
 let write_file ?uuid x lines =
   let fname = get_fname uuid x in
@@ -384,8 +381,8 @@ let write_file ?uuid x lines =
   Lwt_unix.rename fname_new fname
 
 let cleanup_file f =
-  try%lwt Lwt_unix.unlink f
-  with _ -> return_unit
+  try%lwt Lwt_unix.unlink f with
+  | _ -> return_unit
 
 let rmdir dir =
   let command = "rm", [| "rm"; "-rf"; dir |] in
@@ -408,9 +405,26 @@ let unwebize_trustee_public_key pk =
     trustee_public_key = pk.web_trustee_public_key;
   }
 
+let get_suitable_group_kind {t_questions; _} =
+  let group = ref `H in
+  Array.iter (function
+      | Question.NonHomomorphic _ -> group := `NH
+      | Question.Homomorphic _ -> ()
+    ) t_questions;
+  !group
+
+let is_group_fixed se =
+  se.se_public_creds_received
+  || se.se_public_keys <> []
+  || (match se.se_threshold_trustees with
+      | Some l -> l <> []
+      | None -> false
+     )
+
 let default_contact = "Name <user@example.org>"
 
 let default_questions =
+  let open Question_h_t in
   let question = {
       q_answers = [| "Answer 1"; "Answer 2"; "Answer 3" |];
       q_blank = None;
@@ -419,7 +433,7 @@ let default_questions =
       q_question = "Question 1?";
     }
   in
-  [| question |]
+  [| Question.Homomorphic question |]
 
 let default_name = "Name of the election"
 let default_description = "Description of the election."
@@ -434,3 +448,5 @@ let days_to_delete = 365
 let days_to_mail = 30
 let days_between_mails = 7
 let days_to_publish_result = 7
+
+let max_election_name_size = 80
