@@ -120,6 +120,7 @@ type add_account_error =
   | BadUsername
   | BadPassword of string
   | PasswordMismatch
+  | BadSpaceInPassword
 
 val generate_token : ?length:int -> unit -> string Lwt.t
 
@@ -152,6 +153,9 @@ val unurlize : string -> string
 val webize_trustee_public_key : bool -> 'a trustee_public_key -> 'a web_trustee_public_key
 val unwebize_trustee_public_key : 'a web_trustee_public_key -> 'a trustee_public_key
 
+val get_suitable_group_kind : template -> [ `H | `NH ]
+val is_group_fixed : draft_election -> bool
+
 val default_contact : string
 val default_questions : question array
 val default_name : string
@@ -167,3 +171,5 @@ val days_to_delete : int
 val days_to_mail : int
 val days_between_mails : int
 val days_to_publish_result : int
+
+val max_election_name_size : int
