@@ -35,6 +35,9 @@ let enter_cred = "Bitte geben Sie Ihre Wählernummer ein:"
 let invalid_cred = "Falsche Wählernummer!"
 let input_credential = "Wählernummer eingeben"
 let answer_to_questions = "Fragen beantworten"
+let warning_0_255 = "Warning: the system will accept any integer between 0 and 255 but, according to the election rules, invalid ballots (score too high or candidates not properly ranked) will be rejected at the end of the election."
+let alert_0_255 = "Value must be an integer between 0 and 255."
+let at_least_one_invalid = "At least one of the answers is invalid!"
 let review_and_encrypt = "Überprüfen und verschlüsseln"
 let authenticate = "Authentifizieren"
 let confirm = "Bestätigen"
@@ -94,6 +97,7 @@ let login = "Login"
 let password_login = "Login mit Passwort"
 let by_using_you_accept = "Wenn Sie diese Seite benutzen akzeptieren sie unsere "
 let privacy_policy = "Datenschutzbestimmungen"
+let privacy_policy_short = "Datenschutzbestimmungen"
 let accept = "Bestätigen"
 let not_yet_open = "Entschuldigung, die Abstimmung ist noch nicht geöffnet."
 let come_back_later = "Diese Abstimmung gibt es noch nicht. Bitte kommen Sie später wieder."
@@ -109,6 +113,11 @@ let election_archived = "Diese Abstimmung wurde archiviert."
 let number_accepted_ballots = "Anzahl angenommener Stimmen: "
 let you_can_also_download = "Sie können außerdem das "
 let result_with_crypto_proofs = "Ergebnis mit den kryptographischen Beweisen herunterladen"
+let the_raw_results = "The raw results can be viewed in the "
+let json_result = "JSON result"
+let with_the_jsquery = ", with the JavaScript query "
+let it_contains_all_clear = ". It contains all submitted ballots in clear, in random order. It is up to you to apply your favorite counting method (e.g. Condorcet, STV, majority judgement)."
+let applied_shuffles_are = "Applied shuffles are:"
 let blank_vote = "ungültige Stimme"
 let no_other_blank = "Bei einer ungültigen Stimme sind keine anderen Auswahlmöglichkeiten vorhanden."
 let it_will_open_in = "Sie öffnet in "
