@@ -35,6 +35,9 @@ let enter_cred = "Please enter your credential:"
 let invalid_cred = "Invalid credential!"
 let input_credential = "Input credential"
 let answer_to_questions = "Answer to questions"
+let warning_0_255 = "Warning: the system will accept any integer between 0 and 255 but, according to the election rules, invalid ballots (score too high or candidates not properly ranked) will be rejected at the end of the election."
+let alert_0_255 = "Value must be an integer between 0 and 255."
+let at_least_one_invalid = "At least one of the answers is invalid!"
 let review_and_encrypt = "Review and encrypt"
 let authenticate = "Authenticate"
 let confirm = "Confirm"
@@ -94,6 +97,7 @@ let login = "Login"
 let password_login = "Password login"
 let by_using_you_accept = "By using this site, you accept our "
 let privacy_policy = "personal data policy"
+let privacy_policy_short = "Privacy policy"
 let accept = "Accept"
 let not_yet_open = "Sorry, this election is not yet open"
 let come_back_later = "This election does not exist yet. Please come back later."
@@ -109,6 +113,11 @@ let election_archived = "This election is archived."
 let number_accepted_ballots = "Number of accepted ballots: "
 let you_can_also_download = "You can also download the "
 let result_with_crypto_proofs = "result with cryptographic proofs"
+let the_raw_results = "The raw results can be viewed in the "
+let json_result = "JSON result"
+let with_the_jsquery = ", with the JavaScript query "
+let it_contains_all_clear = ". It contains all submitted ballots in clear, in random order. It is up to you to apply your favorite counting method (e.g. Condorcet, STV, majority judgement)."
+let applied_shuffles_are = "Applied shuffles are:"
 let blank_vote = "Blank vote"
 let no_other_blank = "No other choices are allowed when voting blank"
 let it_will_open_in = "It will open in "
