@@ -35,6 +35,9 @@ let enter_cred = "Si prega di inserire il codice di voto :"
 let invalid_cred = "Codice di voto non valido !"
 let input_credential = "Inserire il codice di voto"
 let answer_to_questions = "Risposta alle domande"
+let warning_0_255 = "Warning: the system will accept any integer between 0 and 255 but, according to the election rules, invalid ballots (score too high or candidates not properly ranked) will be rejected at the end of the election."
+let alert_0_255 = "Value must be an integer between 0 and 255."
+let at_least_one_invalid = "At least one of the answers is invalid!"
 let review_and_encrypt = "Riepilogo e cifratura"
 let authenticate = "Autenticazione"
 let confirm = "Conferma"
@@ -94,6 +97,7 @@ let login = "Connettersi"
 let password_login = "Connessione tramite la password"
 let by_using_you_accept = "By using this site, you accept our "
 let privacy_policy = "personal data policy"
+let privacy_policy_short = "Privacy policy"
 let accept = "Accettare"
 let not_yet_open = "Spiacente, quest'elezione non è ancora aperta"
 let come_back_later = "Quest'elezione ancora non esiste. La preghiamo di consultare ulteriormente."
@@ -109,6 +113,11 @@ let election_archived = "Questa elezione è archiviata."
 let number_accepted_ballots = "Numero di schede accettate : "
 let you_can_also_download = "È possibile scaricare il "
 let result_with_crypto_proofs = "risultato con le prove crittografiche"
+let the_raw_results = "The raw results can be viewed in the "
+let json_result = "JSON result"
+let with_the_jsquery = ", with the JavaScript query "
+let it_contains_all_clear = ". It contains all submitted ballots in clear, in random order. It is up to you to apply your favorite counting method (e.g. Condorcet, STV, majority judgement)."
+let applied_shuffles_are = "Applied shuffles are:"
 let blank_vote = "Scheda bianca"
 let no_other_blank = "Nessun'altra scelta è autorizzata quando la scheda è bianca"
 let it_will_open_in = "It will open in "
