@@ -33,7 +33,7 @@ let ( / ) = Filename.concat
 
 let get_draft_election uuid =
   match%lwt read_file ~uuid "draft.json" with
-  | Some [x] -> return @@ Some (draft_election_of_string x)
+  | Some [x] -> return_some (draft_election_of_string x)
   | _ -> return_none
 
 let set_draft_election uuid se =
@@ -41,16 +41,15 @@ let set_draft_election uuid se =
 
 let get_election_result uuid =
   match%lwt read_file ~uuid "result.json" with
-  | Some [x] -> return (Some (election_result_of_string Yojson.Safe.read_json x))
+  | Some [x] -> return_some (election_result_of_string Yojson.Safe.read_json x)
   | _ -> return_none
 
 let set_election_result_hidden uuid hidden =
   match hidden with
   | None ->
-    (
-      try%lwt Lwt_unix.unlink (!Web_config.spool_dir / raw_string_of_uuid uuid / "hide_result")
-      with _ -> return_unit
-    )
+     (try%lwt Lwt_unix.unlink (!Web_config.spool_dir / raw_string_of_uuid uuid / "hide_result") with
+      | _ -> return_unit
+     )
   | Some d -> write_file ~uuid "hide_result" [string_of_datetime d]
 
 let get_election_result_hidden uuid =
@@ -58,7 +57,7 @@ let get_election_result_hidden uuid =
   | Some [x] ->
      let t = datetime_of_string x in
      if datetime_compare (now ()) t < 0 then
-       return (Some t)
+       return_some t
      else
        let%lwt () = set_election_result_hidden uuid None in
        return_none
@@ -129,9 +128,8 @@ let set_election_auto_dates uuid x =
 let set_election_state uuid s =
   match s with
   | `Archived ->
-     (
-       try%lwt Lwt_unix.unlink (!Web_config.spool_dir / raw_string_of_uuid uuid / "state.json")
-       with _ -> return_unit
+     (try%lwt Lwt_unix.unlink (!Web_config.spool_dir / raw_string_of_uuid uuid / "state.json") with
+      | _ -> return_unit
      )
   | _ -> write_file ~uuid "state.json" [string_of_election_state s]
 
@@ -170,7 +168,7 @@ let set_partial_decryptions uuid pds =
 
 let get_decryption_tokens uuid =
   match%lwt read_file ~uuid "decryption_tokens.json" with
-  | Some [x] -> return @@ Some (decryption_tokens_of_string x)
+  | Some [x] -> return_some (decryption_tokens_of_string x)
   | _ -> return_none
 
 let set_decryption_tokens uuid pds =
@@ -179,7 +177,7 @@ let set_decryption_tokens uuid pds =
 
 let get_raw_election uuid =
   match%lwt read_file ~uuid "election.json" with
-  | Some [x] -> return (Some x)
+  | Some [x] -> return_some x
   | _ -> return_none
 
 let empty_metadata = {
@@ -218,7 +216,7 @@ let get_elections_by_owner user =
     Lwt_list.filter_map_s
       (fun x ->
         if x = "." || x = ".." then
-          return None
+          return_none
         else (
           try
             let uuid = uuid_of_raw_string x in
@@ -235,7 +233,7 @@ let get_elections_by_owner user =
                          let election = Election.of_string election in
                          let%lwt kind, date =
                            match%lwt get_election_state uuid with
-                           | `Open | `Closed | `EncryptedTally _ ->
+                           | `Open | `Closed | `Shuffling | `EncryptedTally _ ->
                               let%lwt date = get_election_date `Validation uuid in
                               let date = Option.get date default_validation_date in
                               return (`Validated, date)
@@ -248,16 +246,16 @@ let get_elections_by_owner user =
                               let date = Option.get date default_archive_date in
                               return (`Archived, date)
                          in
-                         return @@ Some (kind, uuid, date, election.e_params.e_name)
+                         return_some (kind, uuid, date, election.e_params.e_name)
                     )
                  | _ -> return_none
                )
             | Some se ->
                if se.se_owner = user then
                  let date = Option.get se.se_creation_date default_creation_date in
-                 return @@ Some (`Draft, uuid, date, se.se_questions.t_name)
+                 return_some (`Draft, uuid, date, se.se_questions.t_name)
                else return_none
-          with _ -> return None
+          with _ -> return_none
         )
       )
 
@@ -278,14 +276,14 @@ let get_passwords uuid =
           SMap.add login (salt, hash) accu
        | _ -> accu
      ) SMap.empty csv in
-     return @@ Some res
+     return_some res
 
 let get_public_keys uuid =
   read_file ~uuid "public_keys.jsons"
 
 let get_private_key uuid =
   match%lwt read_file ~uuid "private_key.json" with
-  | Some [x] -> return (Some (number_of_string x))
+  | Some [x] -> return_some (number_of_string x)
   | _ -> return_none
 
 let get_private_keys uuid =
@@ -293,7 +291,7 @@ let get_private_keys uuid =
 
 let get_threshold uuid =
   match%lwt read_file ~uuid "threshold.json" with
-  | Some [x] -> return (Some x)
+  | Some [x] -> return_some x
   | _ -> return_none
 
 module StringMap = Map.Make (String)
@@ -326,10 +324,12 @@ let get_ballot_hashes uuid =
      StringMap.bindings ballots |> List.map fst |> return
   | _ ->
      let uuid_s = raw_string_of_uuid uuid in
-     let ballots = Lwt_unix.files_of_directory (!Web_config.spool_dir / uuid_s / "ballots") in
-     let%lwt ballots = Lwt_stream.to_list ballots in
-     let ballots = List.filter (fun x -> x <> "." && x <> "..") ballots in
-     return (List.rev_map unurlize ballots)
+     match%lwt Lwt_unix.files_of_directory (!Web_config.spool_dir / uuid_s / "ballots") |> Lwt_stream.to_list with
+     | ballots ->
+        let ballots = List.filter (fun x -> x <> "." && x <> "..") ballots in
+        return (List.rev_map unurlize ballots)
+     | exception Unix.Unix_error(Unix.ENOENT, "opendir", _) ->
+        return []
 
 let get_ballot_by_hash uuid hash =
   match%lwt get_election_state uuid with
@@ -339,7 +339,7 @@ let get_ballot_by_hash uuid hash =
   | _ ->
      let%lwt ballot = read_file ~uuid ("ballots" / urlize hash) in
      match ballot with
-     | Some [x] -> return (Some x)
+     | Some [x] -> return_some x
      | _ -> return_none
 
 let load_ballots uuid =
@@ -349,7 +349,7 @@ let load_ballots uuid =
     let%lwt ballots = Lwt_stream.to_list ballots in
     Lwt_list.filter_map_s (fun x ->
         match%lwt read_file (ballots_dir / x) with
-        | Some [x] -> return (Some x)
+        | Some [x] -> return_some x
         | _ -> return_none
       ) ballots
   ) else return []
@@ -377,22 +377,140 @@ let replace_ballot uuid ~hash ~rawballot =
 let compute_encrypted_tally uuid =
   let%lwt election = get_raw_election uuid in
   match election with
-  | None -> return None
+  | None -> return_none
   | Some election ->
      let election = Election.of_string election in
      let module W = (val Election.get_group election) in
      let module E = Election.Make (W) (LwtRandom) in
      let%lwt ballots = load_ballots uuid in
-     let num_tallied, tally =
-       List.fold_left (fun (n, accu) rawballot ->
-           let ballot = ballot_of_string E.G.read rawballot in
-           let ciphertext = E.extract_ciphertext ballot in
-           n + 1, E.combine_ciphertexts accu ciphertext
-         ) (0, E.neutral_ciphertext ()) ballots
-     in
+     let ballots = Array.map (ballot_of_string E.G.read) (Array.of_list ballots) in
+     let tally = E.process_ballots ballots in
+     let num_tallied = Array.length ballots in
      let tally = string_of_encrypted_tally E.G.write tally in
      let%lwt () = write_file ~uuid (string_of_election_file ESETally) [tally] in
-     return (Some (num_tallied, sha256_b64 tally, tally))
+     return_some (num_tallied, sha256_b64 tally, tally)
+
+let get_shuffle_token uuid =
+  match%lwt read_file ~uuid "shuffle_token.json" with
+  | Some [x] -> return_some (shuffle_token_of_string x)
+  | _ -> return_none
+
+let gen_shuffle_token uuid tk_trustee =
+  let%lwt tk_token = generate_token () in
+  let t = {tk_trustee; tk_token} in
+  let%lwt () = write_file ~uuid "shuffle_token.json" [string_of_shuffle_token t] in
+  return t
+
+let clear_shuffle_token uuid =
+  let f = !Web_config.spool_dir / raw_string_of_uuid uuid / "shuffle_token.json" in
+  try%lwt Lwt_unix.unlink f with _ -> return_unit
+
+let get_nh_ciphertexts uuid =
+  match%lwt get_raw_election uuid with
+  | None -> Lwt.fail (Failure "get_nh_ciphertexts: election not found")
+  | Some election ->
+     let election = Election.of_string election in
+     let module W = (val Election.get_group election) in
+     let module E = Election.Make (W) (LwtRandom) in
+     let%lwt current =
+       match%lwt read_file ~uuid "shuffles.jsons" with
+       | None -> return []
+       | Some x -> return x
+     in
+     match List.rev current with
+     | [] ->
+        let%lwt tally =
+          match%lwt read_file ~uuid (string_of_election_file ESETally) with
+          | Some [x] -> return (encrypted_tally_of_string E.G.read x)
+          | _ -> Lwt.fail (Failure "get_nh_ciphertexts: encrypted tally not found or invalid")
+        in
+        return (string_of_nh_ciphertexts E.G.write (E.extract_nh_ciphertexts tally))
+     | x :: _ ->
+        let s = shuffle_of_string E.G.read x in
+        return (string_of_nh_ciphertexts E.G.write s.shuffle_ciphertexts)
+
+let get_shuffles uuid =
+  match%lwt get_raw_election uuid with
+  | None -> return_none
+  | Some _ ->
+     match%lwt read_file ~uuid "shuffles.jsons" with
+     | None -> return_none
+     | Some x ->
+        let rec loop accu = function
+          | s :: rest -> loop (s :: accu) rest
+          | [] -> return_some (List.rev accu)
+        in
+        loop [] x
+
+let get_shuffle_hashes uuid =
+  match%lwt read_file ~uuid "shuffle_hashes.jsons" with
+  | None -> return_none
+  | Some x ->
+     let rec loop accu = function
+       | s :: rest -> loop (shuffle_hash_of_string s :: accu) rest
+       | [] -> return_some (List.rev accu)
+     in
+     loop [] x
+
+let add_shuffle_hash uuid sh =
+  let%lwt current =
+    match%lwt get_shuffle_hashes uuid with
+    | None -> return []
+    | Some x -> return x
+  in
+  let () =
+    if List.exists (fun x -> x.sh_trustee = sh.sh_trustee) current then (
+      Printf.ksprintf failwith "add_shuffle_hash(%s, %s): existing trustee"
+        (raw_string_of_uuid uuid) sh.sh_trustee
+    )
+  in
+  let new_ = current @ [sh] in
+  write_file ~uuid "shuffle_hashes.jsons" (List.map string_of_shuffle_hash new_)
+
+let compute_encrypted_tally_after_shuffling uuid =
+  let%lwt election = get_raw_election uuid in
+  match election with
+  | None -> return_none
+  | Some election ->
+     let election = Election.of_string election in
+     let module W = (val Election.get_group election) in
+     let module E = Election.Make (W) (LwtRandom) in
+     match%lwt read_file ~uuid (string_of_election_file ESETally) with
+     | Some [x] ->
+        let tally = encrypted_tally_of_string E.G.read x in
+        let%lwt nh = get_nh_ciphertexts uuid in
+        let nh = nh_ciphertexts_of_string E.G.read nh in
+        let tally = E.merge_nh_ciphertexts nh tally in
+        let tally = string_of_encrypted_tally E.G.write tally in
+        let%lwt () = write_file ~uuid (string_of_election_file ESETally) [tally] in
+        return_some (sha256_b64 tally, tally)
+     | _ -> return_none
+
+let shuffle_mutex = Lwt_mutex.create ()
+
+let append_to_shuffles uuid shuffle =
+  match%lwt get_raw_election uuid with
+  | None -> Lwt.fail (Failure "append_to_shuffles: election not found")
+  | Some election ->
+     let election = Election.of_string election in
+     let module W = (val Election.get_group election) in
+     let module E = Election.Make (W) (LwtRandom) in
+     let shuffle = shuffle_of_string E.G.read shuffle in
+     Lwt_mutex.with_lock shuffle_mutex (fun () ->
+         let%lwt last_ciphertext = get_nh_ciphertexts uuid in
+         let last_ciphertext = nh_ciphertexts_of_string E.G.read last_ciphertext in
+         if E.check_shuffle last_ciphertext shuffle then (
+           let%lwt current =
+             match%lwt read_file ~uuid "shuffles.jsons" with
+             | None -> return []
+             | Some x -> return x
+           in
+           let shuffle_ = string_of_shuffle E.G.write shuffle in
+           let new_ = current @ [shuffle_] in
+           let%lwt () = write_file ~uuid "shuffles.jsons" new_ in
+           return_some (sha256_b64 shuffle_)
+         ) else return_none
+       )
 
 module ExtendedRecordsCacheTypes = struct
   type key = uuid
